#include <algorithm>
#include <fstream>
#include <iostream>
#include <regex>
#include <string>
namespace {
  int zaehleLOC(const std::string& dateiname)
  {
    std::ifstream quelle(dateiname, std::ios::binary);
    if (!quelle.good()) {
      throw std::ios::failure("Dateifehler");
    }
    std::string alles;
    while (quelle.good()) {
      char c = static_cast<char>(quelle.get());
      if (!quelle.fail()) {
        alles += c;
      }
    }
    // Zeilenendekennung vereinheitlichen
    alles = std::regex_replace(alles, std::regex("(\r\n)"), "\n");
    // Leer- und Tabulatorzeichen entfernen
    alles = std::regex_replace(alles, std::regex("( |\t)"), "");
    // Escape-Zeichen loeschen
    alles = std::regex_replace(alles, std::regex("\\\\."), "");
    // Alle Strings durch "" nicht-greedy ersetzen
    alles = std::regex_replace(alles, std::regex("\".*?\""), "\"\"");
    // alle // Kommentare loeschen
    alles = std::regex_replace(alles, std::regex("//[^\n]*"), "");
    /* So  beginnende Kommentare zeilenübergreifend löschen. Im folgenden regulären
       Ausdruck bedeutet das erste ?, dass der Inhalt der Capturing Group nicht
       gespeichert werden muss. Das zweite ? bedeutet 'nicht-greedy'. Ohne das würde bis 
       zum Ende des letzten zeilenübergreifenden Kommentars gelöscht werden.   */
    alles = std::regex_replace(alles, std::regex("/\\*(?:.|\n)*?\\*/"), "");
    // Leerzeilen entfernen
    alles = std::regex_replace(alles, std::regex("(\\n)+"), "$1");
    // Anzahl der Zeilen zurückgeben
    return 1 + std::count(alles.begin(), alles.end(), '\n');
  }
}                                            // namespace
int main(int argc, char* argv[])
{
  if (2 != argc) {
    std::cout << "Gebrauch: " << argv[0] << " dateiname\n";
    return 1;                                // EXIT
  }
  // Datei analysieren
  try {
    std::cout << "Die Datei " << argv[1] << " hat " << zaehleLOC(argv[1])
              << " Lines of Code (LOC).\n";
  }
  catch (std::regex_error& re) {
    std::cerr << "Regex-Fehler: " << re.what() << '\n';
  }
  catch (std::ios::failure& e) {
    std::cerr << e.what() << '\n';
  }
}
