#ifndef ARRAY2D_H
#define ARRAY2D_H
#include <iostream>
#include <stdexcept>
#include <string>
#include <vector>

template <typename T>
class Array2d {
public:
  Array2d(int z, int s, const T& wert = T())
  : zeilen_{z}, spalten_{s}, daten{std::vector<T>(z * s, wert)}
  { }

  auto zeilen() const { return zeilen_; }
  auto spalten() const { return spalten_; }

  void check_indizes(int z, int s) const
  {
    const std::string datei {__FILE__};
    if (z < 0 || z >= zeilen_) {
      throw std::range_error(datei +": z=" + std::to_string(z) + " ungültig!");
    }
    if (s < 0 || s >= spalten_) {
      throw std::range_error(datei +": s=" + std::to_string(s) + " ungültig!");
    }
  }

  const auto& operator()(int z, int s) const 
  { 
#ifdef INDEXCHECK
    check_indizes(z, s);
#endif
    return daten[+ z * spalten_ + s]; 
  }
  auto& operator()(int z, int s) 
  {
#ifdef INDEXCHECK
    check_indizes(z, s);
#endif
    return daten[+ z * spalten_ + s]; 
  }

  void init(const T& wert) 
  {
    for (auto& element : daten) {
      element = wert;
    }
  }

  // Multiplikation mit einer Matrix b
  auto& operator*=(const Array2d<T>& b)
  {
    if (spalten() != b.zeilen()) {
      throw "Falsche Dimension in Matrix*= !\n";
    }
    Array2d<T> erg(zeilen(), b.spalten());
    for (int i = 0; i < zeilen(); ++i) {
      for (int j = 0; j < b.spalten(); ++j) {
        erg(i, j) = T(0);
        for (int k = 0; k < spalten(); ++k)
          erg(i, j) += operator()(i, k) * b(k, j);
      }
    }
    std::swap(*this, erg);                     // vertauschen
    return *this;
  }
  // Für eine Alternative des Gleichheitsoperators, s.u.
  const auto& get_daten() const { return daten; }
private:
  int zeilen_;
  int spalten_;
  std::vector<T> daten;
};

template <typename T>
void print(const Array2d<T>& a, 
           std::ostream& ausgabe = std::cout) 
{
  for (int z = 0; z < a.zeilen(); ++z) {
    for (int s = 0; s < a.spalten(); ++s) {
      ausgabe << a(z, s) << " ";
    }
    ausgabe << '\n';
  }
  ausgabe << '\n';
}

// binäre Multiplikation
template <typename T> 
Array2d<T> operator*(Array2d<T> a, const Array2d<T>& b) { return a *= b; }
/*
// Vergleich, falls get_daten() nicht existiert
template <typename T> 
bool operator==(const Array2d<T>& a, const Array2d<T>& b) 
{
  if (a.zeilen() != b.zeilen()) return false;
  if (a.spalten() != b.spalten()) return false;
  for (int i = 0; i < a.zeilen(); ++i) {
    for (int j = 0; j < a.spalten(); ++j) {
      if (a(i, j) != b(i, j))
        return false;
    }
  }
  return true;
}
*/
template <typename T> 
bool operator==(const Array2d<T>& a, const Array2d<T>& b) 
{
  return a.get_daten() == b.get_daten();
}
/* weitere Alternativen:
1. operator==(const Array2d<T>&) als Member
2. operator==(const Array2d<T>& a, const Array2d<T>& b) als friend
*/
#endif
